import numpy as np
import pandas as pd


from eradication_data_requirements.progress_probability import get_progress_probability
from eradication_data_requirements.calculate_intersect import add_cpue
from eradication_data_requirements.data_requirements_plot import fit_ramsey_plot
from eradication_data_requirements.resample_raw_data import (
    resample_valid_data,
    resample_valid_cumulative_data,
)
from eradication_data_requirements.set_data import select_month_by_window_length


def fit_resampled_captures(datos, bootstrapping_number):
    resampled_data = resample_valid_data(datos, bootstrapping_number)
    ramsey_series = [set_up_ramsey_time_series(sample) for sample in resampled_data]
    fits = [fit_ramsey_plot(ramsey_serie) for ramsey_serie in ramsey_series]
    return fits


def add_probs_to_effort_capture_data(data_copy, bootstrapping_number, window_length):
    resized_data = data_copy[data_copy.Esfuerzo != 0]
    complete_months_data = fill_missing_months_with_effort_one_and_captures_zero(resized_data)
    data_with_cpue = add_cpue(complete_months_data)
    probs_status = calculate_resampled_probability_by_window(
        data_with_cpue, bootstrapping_number, window_length
    )
    data_with_cpue = paste_status_by_window(data_with_cpue, probs_status, "prob", window_length)
    data_with_cpue.dropna(subset=["prob"], inplace=True)
    return data_with_cpue[["Fecha", "Esfuerzo", "Capturas", "prob"]]


def fill_missing_months_with_effort_one_and_captures_zero(effort_and_capture_data):
    data_copy = effort_and_capture_data.copy()
    data_with_all_months = complete_missing_months_in_year(data_copy)
    return fill_empty_months_with_effort_one_and_captures_zero(data_with_all_months)


def complete_missing_months_in_year(incomplete_data):
    incomplete_data["Fecha"] = pd.to_datetime(incomplete_data["Fecha"], format="%Y-%m-%d")
    incomplete_data = incomplete_data.set_index("Fecha")
    initial_year = pd.offsets.YearBegin()
    one_day = pd.offsets.Day()
    date_index = pd.date_range(
        incomplete_data.index.min() + one_day - initial_year, incomplete_data.index.max(), freq="MS"
    )
    filled_months_df = incomplete_data.reindex(date_index, fill_value=np.nan)
    filled_months_df["Fecha"] = filled_months_df.index.strftime("%Y-%m-%d").astype(str)
    return filled_months_df


def fill_empty_months_with_effort_one_and_captures_zero(data_copy):
    data_copy["Esfuerzo"] = data_copy["Esfuerzo"].fillna(1)
    data_copy["Capturas"] = data_copy["Capturas"].fillna(0)
    return data_copy.reset_index()


def paste_status_by_window(data_copy, probs_status, column_name, window_length):
    df = add_empty_column(data_copy, column_name)
    indexes_with_probability = select_month_by_window_length(data_copy, window_length)
    df.loc[indexes_with_probability.unique(), column_name] = probs_status
    return df


def add_empty_column(data_copy, column_name):
    df_copy = data_copy.copy()
    df_copy.loc[:, column_name] = np.nan
    return df_copy


def set_up_ramsey_time_series(data):
    cumulative_captures = pd.DataFrame()
    cumulative_captures["Fecha"] = data.Fecha
    cumulative_captures["Cumulative_captures"] = data["Capturas"].cumsum()
    cumulative_captures["CPUE"] = data["Capturas"] / data["Esfuerzo"]
    return cumulative_captures[["Fecha", "CPUE", "Cumulative_captures"]]


def fit_resampled_cumulative(datos, bootstrapping_number):
    ramsey_series = set_up_ramsey_time_series(datos)
    resampled_data = resample_valid_cumulative_data(ramsey_series, bootstrapping_number)
    fits = [fit_ramsey_plot(sample) for sample in resampled_data]
    return fits


def calculate_resampled_probability_by_window(ramsey_series, bootstrapping_number, window_length):
    seed = 42
    indexes_to_resample = select_month_by_window_length(ramsey_series, window_length)
    return [
        get_progress_probability(
            ramsey_series.loc[(i - window_length) : i], bootstrapping_number, seed
        )
        for i in indexes_to_resample
    ]


def calculate_six_months_slope(data):
    window_length = 6
    return [
        fit_ramsey_plot(data.iloc[(i - window_length) : i])
        for i in range(window_length, len(data) + 1)
    ]


def extract_slopes(slopes_intercept_data):
    return [slope[0] for slope in slopes_intercept_data]


def extract_prob(slopes_intercept_data):
    slopes = [np.asarray(extract_slopes(sample)) for sample in slopes_intercept_data]
    return [np.mean(samples < 0) for samples in slopes]



from bootstrapping_tools import AbstractSeriesBootstrapper
from eradication_data_requirements.data_requirements_plot import fit_ramsey_plot
from eradication_data_requirements.resample_raw_data import resample_eradication_data

import numpy as np
import json


class ProgressBootstrapper(AbstractSeriesBootstrapper):
    def __init__(self, bootstrapper_parametrizer):
        bootstrapper_parametrizer.parameters["dataframe"]["CPUE"] = 1
        self.bootstrap_config = bootstrapper_parametrizer.parameters
        self.season_series = self.bootstrap_config["dataframe"]["Temporada"]
        self.data_series = self.add_cpue()
        self.parameters_distribution = self.get_parameters_distribution()

    def add_cpue(self):
        data = self.bootstrap_config["dataframe"]
        data["CPUE"] = data.Capturas / data.Esfuerzo
        return data

    def get_parameters_distribution(self):
        rng = np.random.default_rng(42)
        distribution = []
        distribution_size = 0
        captures = self.data_series.Capturas.sum()
        while self.bootstrap_config["N"] > distribution_size:
            sample = resample_eradication_data(self.data_series, rng)
            parameters = fit_ramsey_plot(sample)
            is_valid = -parameters[1] / parameters[0] > captures
            if is_valid:
                distribution.append((parameters[0], parameters[1]))
            distribution_size = len(distribution)
        return distribution

    def save_intervals(self, output_path):
        json_dict = self.get_parameters_dictionary()
        json_dict["slopes_latex_interval"] = json_dict.pop("main_parameter_latex_interval")
        with open(output_path, "w") as file:
            json.dump(json_dict, file)



from bootstrapping_tools import generate_latex_interval_string
from eradication_data_requirements.calculate_intersect import get_confidence_interval

import numpy as np


def combine_distributions_from_dict(pop_status_a, pop_status_b):
    capturas_totales = pop_status_a["capturas"] + pop_status_b["capturas"]
    mixed_distribution = concatenate_remanent_distributions(pop_status_a, pop_status_b)
    remanents_interval = get_confidence_interval(mixed_distribution)
    remanents_interval_string = generate_latex_interval_string(
        remanents_interval, deltas=False, decimals=0
    )
    return {
        "remanentes": remanents_interval_string,
        "capturas": capturas_totales,
        "remanentes_distribution": list(mixed_distribution),
    }


def concatenate_remanent_distributions(pop_status_a, pop_status_b):
    remanents_a = [n0 - pop_status_a["capturas"] for n0 in pop_status_a["distribution"]]
    remanents_b = [n0 - pop_status_b["capturas"] for n0 in pop_status_b["distribution"]]
    return np.concatenate((remanents_a, remanents_b))



from geci_plots import roundup, order_magnitude
import numpy as np
import matplotlib.pyplot as plt


def plot_cumulative_series_cpue(fontsize, cpue_df):
    seasons = cpue_df.index.values
    seasons_labels = [*seasons, ""]
    ticks_positions = np.arange(seasons[0], seasons[-1] + 2)
    ticks_positions[-1] = ticks_positions[-1] + 0.25

    _, ax = plt.subplots(1, 2, figsize=(23, 10), tight_layout=True)

    ax[0].plot(seasons, cpue_df["cpue"], "-o", linewidth=2)
    ax[0].set_xticks(ticks_positions)
    ax[0].set_xticklabels(seasons_labels, size=fontsize)
    ax[0].tick_params(axis="both", labelsize=fontsize)
    ax[0].spines["right"].set_visible(False)
    ax[0].spines["top"].set_visible(False)
    max_cpue = max(cpue_df["cpue"])
    cpue_limit = roundup(max_cpue, 10 ** order_magnitude(max_cpue))
    ax[0].set_ylim(0, cpue_limit)
    ax[0].set_ylabel("Catch Per Unit Effort (CPUE)", fontsize=fontsize)
    ax[0].set_xlim(ticks_positions[0] - 1, ticks_positions[-1])

    ax[1].plot(seasons, cpue_df["cumulative_cpue"], "-o", linewidth=2)
    ax[1].set_xticks(ticks_positions)
    ax[1].set_xticklabels(seasons_labels, size=fontsize)
    ax[1].tick_params(axis="both", labelsize=fontsize)
    ax[1].spines["right"].set_visible(False)
    ax[1].spines["top"].set_visible(False)
    max_cum_cpue = max(cpue_df["cumulative_cpue"])
    cum_cpue_limit = roundup(max_cum_cpue, 10 ** order_magnitude(max_cum_cpue))
    ax[1].set_ylim(0, cum_cpue_limit)
    ax[1].set_ylabel("Cumulative CPUE", fontsize=fontsize)
    ax[1].set_xlim(ticks_positions[0] - 1, ticks_positions[-1])
    return ax


def calculate_cpue_and_cumulative_by_season(effort_capture_df):
    extract_year(effort_capture_df)
    effort_capture_df = effort_capture_df[effort_capture_df["Season"] >= 2014]
    column_name = "Season"
    return calculate_cpue_and_cumulative_by_column(effort_capture_df, column_name)


def calculate_cpue_and_cumulative_by_flight(effort_capture_df):
    column_name = "No_vuelo"
    return calculate_cpue_and_cumulative_by_column(effort_capture_df, column_name)


def calculate_cpue_and_cumulative_by_column(effort_capture_df, column_name):
    data_grouped_by_column = effort_capture_df.groupby(by=column_name).sum(numeric_only=False)
    data_grouped_by_column["cpue"] = (
        data_grouped_by_column["Capturas"] / data_grouped_by_column["Esfuerzo"]
    )
    data_grouped_by_column["cumulative_cpue"] = data_grouped_by_column["cpue"].cumsum()
    return data_grouped_by_column


def extract_year(effort_capture_df):
    effort_capture_df["Season"] = effort_capture_df["Fecha"].str[:4]
    effort_capture_df["Season"] = np.array([int(season) for season in effort_capture_df["Season"]])



from eradication_data_requirements.plot_cpue_series import (
    calculate_cpue_and_cumulative_by_flight,
    plot_cumulative_series_cpue,
)

import pandas as pd
import typer
import matplotlib.pyplot as plt

app = typer.Typer()


@app.command()
def version():
    pass


@app.command()
def plot_cumulative_series_cpue_by_flight(
    effort_capture_path: str = typer.Option("", help="Input file path"),
    output_png: str = typer.Option("", help="Output file path"),
    fontsize: int = typer.Option(27, help="Font size of axis"),
):
    effort_capture_df = pd.read_csv(effort_capture_path)
    data_year = calculate_cpue_and_cumulative_by_flight(effort_capture_df)
    plot_cumulative_series_cpue(fontsize, data_year)
    plt.savefig(output_png, dpi=300, transparent=True)



import numpy as np
import matplotlib.pyplot as plt
import warnings

from geci_plots import geci_plot


def fit_ramsey_plot(data):
    try:
        fit = np.polynomial.polynomial.Polynomial.fit(
            data["Cumulative_captures"], data["CPUE"], deg=1
        )
        intercept_and_slope = fit.convert().coef
        idx = [1, 0]
        slope_and_intercept = intercept_and_slope[idx]
    except (AssertionError, IndexError):
        warnings.warn("Error")
        slope_and_intercept = [np.nan, np.nan]
    return slope_and_intercept


def plot_comparative_catch_curves(socorro_data, guadalupe_data):
    _, ax = geci_plot()
    ax = plot_catch_curve(socorro_data, ax, "Socorro")
    ax = plot_catch_curve(guadalupe_data, ax, "Guadalupe")
    plt.xlabel("Cumulative captures", size=15, labelpad=15)
    plt.ylabel("CPUE (captures/night traps)", size=15)
    plt.legend(fontsize="xx-large")


def plot_traps_data_requirements(data):
    config_plot = {"ylabel": "CPUE (captures/night traps)", "cumulative": "Cumulative_captures"}
    return data_requirements_plot(data, config_plot)


def data_requirements_plot(data, config_plot):
    configured_data = set_cumulative_captures_column(data, config_plot)
    _, ax = geci_plot()
    ax = plot_catch_curve(configured_data, ax)
    plt.xlabel("Cumulative captures", size=15, labelpad=15)
    plt.ylabel(config_plot["ylabel"], size=15)
    return ax


def set_cumulative_captures_column(data, config):
    return data.rename(columns=config)


def plot_catch_curve(data, ax, label=None):
    theta = fit_ramsey_plot(data)
    y_line = theta[1] + theta[0] * data["Cumulative_captures"]
    ax.plot(data["Cumulative_captures"], y_line, "r")
    ax.scatter(data["Cumulative_captures"], data["CPUE"], marker="o", label=label)
    return ax



import numpy as np
from bootstrapping_tools import generate_latex_interval_string
from eradication_data_requirements.data_requirements_plot import fit_ramsey_plot
from eradication_data_requirements.progress_probability import get_progress_probability
from eradication_data_requirements.resample_raw_data import resample_eradication_data


def get_population_status_dict(raw_data, bootstrap_number, seed):
    data_with_cpue = add_cpue(raw_data)
    intercepts_distribution = get_intercepts_distribution(data_with_cpue, bootstrap_number, seed)
    interval = get_confidence_interval(intercepts_distribution)
    n0_interval = generate_latex_interval_string(interval, deltas=False, decimals=0)

    captures = data_with_cpue.Capturas.sum()
    remanentes = interval - captures
    remanentes_interval = generate_latex_interval_string(remanentes, deltas=False, decimals=0)
    progress_probability = get_progress_probability(data_with_cpue, bootstrap_number, seed)
    json_content = {
        "n0": n0_interval,
        "remanentes": remanentes_interval,
        "capturas": int(captures),
        "progress_probability": progress_probability,
        "distribution": intercepts_distribution,
    }
    return json_content


def add_cpue(raw_data):
    raw_data["CPUE"] = raw_data.Capturas / raw_data.Esfuerzo
    return raw_data


def get_confidence_interval(distribution):
    interval = np.round(np.percentile(distribution, [2.5, 50, 97.5])).astype(int)
    return interval


def get_intercepts_distribution(raw_data, bootstrap_number, seed=None):
    rng = np.random.default_rng(seed)
    raw_distribution = []
    distribution_size = 0
    captures = raw_data.Capturas.sum()
    while distribution_size < bootstrap_number:
        intercept = calculate_x_intercept(resample_eradication_data(raw_data, rng))
        if intercept > captures:
            raw_distribution.append(intercept)
        distribution_size = len(raw_distribution)
    return raw_distribution


def calculate_x_intercept(data):
    parameters = fit_ramsey_plot(data)
    return -parameters[1] / parameters[0]



from bootstrapping_tools import Bootstrap_from_time_series_parametrizer
from eradication_data_requirements.calculate_eradication_progress import ProgressBootstrapper
from eradication_data_requirements.calculate_intersect import get_population_status_dict
from eradication_data_requirements.cli import plot_cumulative_series_cpue_by_flight
from eradication_data_requirements.data_requirements_plot import (
    plot_traps_data_requirements,
    plot_comparative_catch_curves,
    data_requirements_plot,
)
from eradication_data_requirements.mix_distributions import combine_distributions_from_dict
from eradication_data_requirements.plot_progress_probability import plot_progress_probability
from eradication_data_requirements.resample_aerial_monitoring import get_monitoring_dict
from eradication_data_requirements.set_data import filter_data_by_method
from eradication_data_requirements.fit_ramsey_time_series import add_probs_to_effort_capture_data

from fastapi import FastAPI, UploadFile, File, Form
from fastapi.responses import (
    JSONResponse,
    StreamingResponse,
)
import io
import json
import matplotlib.pyplot as plt
import pandas as pd

api = FastAPI()


@api.get("/write_bootstrap_progress_intervals_json")
async def write_bootstrap_progress_intervals_json(
    input_path: str, bootstrapping_number: int, output_path: str
):
    parametrizer = Bootstrap_from_time_series_parametrizer(
        blocks_length=1,
        column_name="CPUE",
        N=bootstrapping_number,
        independent_variable="Capturas",
    )
    data = pd.read_csv(input_path)
    parametrizer.set_data(data)
    bootstrapper = ProgressBootstrapper(parametrizer)
    bootstrapper.save_intervals(output_path)


@api.get("/write_aerial_monitoring")
async def api_write_aerial_monitoring(input_path: str, bootstrapping_number: int, output_path: str):
    raw_data = pd.read_csv(input_path)
    json_content = get_monitoring_dict(raw_data, bootstrapping_number)
    write_json(output_path, json_content)


@api.get("/filter_by_method")
async def api_filter_by_method(input_path: str, method: str, output_path: str):
    raw_data = pd.read_csv(input_path)
    filtered_data = filter_data_by_method(raw_data, method)
    filtered_data.to_csv(output_path, index=False)


@api.get("/write_population_status_from_mixed_methods")
async def api_write_population_status_from_mixed_methods(
    first_method_status: str, second_method_status: str, output_path: str
):
    first_status_dict = read_json(first_method_status)
    second_status_dict = read_json(second_method_status)
    json_content = combine_distributions_from_dict(first_status_dict, second_status_dict)
    write_json(output_path, json_content)


@api.get("/plot_cumulative_series_cpue_by_flight")
async def api_plot_cumulative_series_cpue_by_flight(input_path: str, output_path: str):
    font_size = 27
    plot_cumulative_series_cpue_by_flight(input_path, output_path, font_size)


@api.post("/write_population_status")
async def api_write_population_status(
    file: UploadFile = File(...),
    bootstrapping_number: int = Form(...),
):
    raw_data = pd.read_csv(file.file)
    seed = 42
    json_content = get_population_status_dict(raw_data, bootstrapping_number, seed)
    return JSONResponse(content=json_content)


@api.post("/write_effort_and_captures_with_probability")
async def api_write_effort_and_captures_with_probability(
    file: UploadFile = File(...),
    bootstrapping_number: int = Form(...),
    window_length: int = Form(...),
):
    effort_capture_data = pd.read_csv(file.file)
    effort_captures_with_slopes = add_probs_to_effort_capture_data(
        effort_capture_data, bootstrapping_number, window_length
    )
    yearly_json = effort_captures_with_slopes.to_dict(orient="records")
    return JSONResponse(content=yearly_json)


@api.post("/write_probability_figure")
async def api_write_probability_figure(file: UploadFile = File(...)):
    monthly_progress_probability = pd.read_csv(file.file)
    plot_progress_probability(monthly_progress_probability)
    buffer = io.BytesIO()
    plt.savefig(buffer, format="png", bbox_inches="tight")
    buffer.seek(0)
    plt.close()
    return StreamingResponse(buffer, media_type="image/png")


@api.post("/plot_custom_cpue_vs_cum_captures")
async def api_plot_custom_cpue_vs_cum_captures(
    file: UploadFile = File(...), config: UploadFile = File(...), format: str = Form("png")
):
    config_plot = json.load(config.file)
    data = pd.read_csv(file.file)
    data_requirements_plot(data, config_plot)
    return save_figure_as_buffer(format)


@api.post("/plot_cpue_vs_cum_captures")
async def api_plot_cpue_vs_cum_captures(file: UploadFile = File(...), format: str = Form("png")):
    cumulative_effort_and_captures_data = pd.read_csv(file.file)
    plot_traps_data_requirements(cumulative_effort_and_captures_data)
    return save_figure_as_buffer(format)


@api.post("/plot_comparative_catch_curves")
async def api_plot_comparative_catch_curves(
    socorro_file: UploadFile = File(...),
    guadalupe_file: UploadFile = File(...),
    format: str = Form("png"),
):
    socorro_data = pd.read_csv(socorro_file.file)
    guadalupe_data = pd.read_csv(guadalupe_file.file)
    plot_comparative_catch_curves(socorro_data, guadalupe_data)
    return save_figure_as_buffer(format)


def read_json(json_path):
    with open(json_path) as json_file:
        data = json.load(json_file)
    return data


def write_json(output_path, json_content):
    with open(output_path, "w") as jsonfile:
        json.dump(json_content, jsonfile)


def save_figure_as_buffer(format):
    buffer = io.BytesIO()
    plt.savefig(buffer, dpi=300, transparent=True, format=format)
    buffer.seek(0)
    plt.close()
    return StreamingResponse(buffer, media_type=f"image/{format}")



import geci_plots as gp
import matplotlib.pyplot as plt


def plot_progress_probability(data):
    _, ax = gp.geci_plot()

    eradication = 0.80
    control = 0.50

    ax.fill_between(data.Fecha, 1, eradication, facecolor="green", alpha=0.5)
    ax.fill_between(data.Fecha, eradication, control, facecolor="yellow", alpha=0.5)
    ax.fill_between(data.Fecha, control, 0, facecolor="red", alpha=0.5)
    ax.scatter(data.Fecha, data.prob, color="black")
    ax.plot(data.Fecha, data.prob, color="black", alpha=0.3)
    locs, labels = plt.xticks()

    labels = [item.get_text()[:7] for item in labels]
    plt.xticks(locs, labels, rotation=90)
    plt.ylabel("Progress probability", fontsize=20)

    return ax



"""A template Python module"""

__version__ = "6.0.0"
from .api import *  # noqa
from .calculate_eradication_progress import *  # noqa
from .calculate_intersect import *  # noqa
from .data_requirements_plot import *  # noqa
from .fit_ramsey_time_series import *  # noqa
from .plot_cpue_series import *  # noqa
from .plot_progress_probability import *  # noqa
from .resample_aerial_monitoring import *  # noqa
from .resample_raw_data import *  # noqa
from .set_data import *  # noqa



import numpy as np
import warnings

from eradication_data_requirements.data_requirements_plot import fit_ramsey_plot
from eradication_data_requirements.resample_raw_data import resample_eradication_data


def get_progress_probability(raw_data, bootstrap_number, seed):
    slope_distribution = get_slopes_distribution(raw_data, bootstrap_number, seed)
    progress_probability = calculate_progress_probability(slope_distribution)
    return np.round(progress_probability, decimals=3)


def calculate_progress_probability(slopes_distribution):
    valid_slopes = [valid_slope for valid_slope in slopes_distribution if not np.isnan(valid_slope)]
    slopes_len = len(valid_slopes)
    is_in_progress = valid_slopes < np.zeros(slopes_len)
    return sum(is_in_progress) / slopes_len


def get_slopes_distribution(raw_data, bootstrap_number, seed):
    rng = np.random.default_rng(seed)
    slopes_distribution = [
        get_slope(resample_eradication_data(raw_data, rng)) for _ in range(bootstrap_number)
    ]
    if np.isnan(slopes_distribution).all():
        raise TypeError("Empty slopes distribution")
    return slopes_distribution


def get_slope(data):
    try:
        parameters = fit_ramsey_plot(data)
    except AssertionError as error:
        warnings.warn(error)
        parameters = [np.nan]
    return parameters[0]



from bootstrapping_tools import resample_data, generate_latex_interval_string
from .calculate_intersect import get_confidence_interval


def get_monitoring_dict(raw_data, bootstrap_number):
    distribution = get_sum_distribution(raw_data, bootstrap_number)
    interval = get_confidence_interval(distribution)
    total_interval = generate_latex_interval_string(interval, deltas=False, decimals=0)
    return {"total": total_interval}


def get_sum_distribution(df, bootstrap_number):
    blocks_length = 1
    return [
        int(resample_data(df, seed, blocks_length).No_goats.sum())
        for seed in range(bootstrap_number)
    ]



from bootstrapping_tools import resample_data


def resample_eradication_data(data, rng):
    resampled_data = data.sample(replace=True, frac=1, random_state=rng)
    sorted_data = resampled_data.sort_index()
    sorted_data["Cumulative_captures"] = sorted_data.Capturas.cumsum()
    return sorted_data[["CPUE", "Cumulative_captures"]]


def resample_valid_data(effort_and_capture_data, bootstrapping_number):
    validate_method = validate_samples_to_fit
    return resample_valid_data_by_method(
        effort_and_capture_data, bootstrapping_number, validate_method
    )


def resample_valid_cumulative_data(cumulative_captures_data, bootstrapping_number):
    validate_method = validate_cumulative_samples_to_fit
    return resample_valid_data_by_method(
        cumulative_captures_data, bootstrapping_number, validate_method
    )


def resample_valid_data_by_method(effort_and_capture_data, bootstrapping_number, validate_method):
    blocks_length = 2
    sample = generate_samples(effort_and_capture_data, bootstrapping_number, blocks_length)
    return validate_method(sample)


def generate_samples(cumulative_captures_data, bootstrapping_number, blocks_length):
    return [
        resample_data(cumulative_captures_data, seed, blocks_length)
        for seed in range(bootstrapping_number)
    ]


def validate_samples_to_fit(samples):
    validated = [
        valid_sample
        for valid_sample in samples
        if valid_sample.Capturas.sum() != valid_sample.Capturas.iloc[0]
    ]
    return validated


def validate_cumulative_samples_to_fit(samples):
    validated = [
        valid_sample
        for valid_sample in samples
        if len(valid_sample.Cumulative_captures.unique()) > 1
        and len(valid_sample.CPUE.unique()) > 1
    ]
    return validated



import pandas as pd
from functools import reduce


def filter_data_by_method(raw_data, method):
    filtered_by_method = raw_data[raw_data.Tecnica == method]
    filtered_by_method.loc[:, ["Acumulado"]] = filtered_by_method.Capturas.cumsum()
    return filtered_by_method


def select_month_by_window_length(data, window_length):
    month_to_plot = [f"-{factor:02d}-" for factor in range(1, 13) if factor % window_length == 0]
    mask = reduce(
        lambda x, y: x | y,
        [data["Fecha"].str.contains(pattern, case=False) for pattern in month_to_plot],
    )
    cutted_months = data[mask]
    return pd.concat([cutted_months, data.iloc[-1:]]).index
